@@ -3,7 +3,7 @@ use warnings;
 
 package Template::Pure;
 
-our $VERSION = '0.012';
+our $VERSION = '0.013';
 
 use DOM::Tiny;
 use Scalar::Util;
@@ -30,6 +30,9 @@ sub new {
 sub render {
   my ($self, $data_proto, $extra_directives) = @_;
   my $dom = DOM::Tiny->new($self->{template});
+  
+  $data_proto = Template::Pure::DataProxy->new($data_proto, self=>$self);
+
   return $self->process_dom($dom, $data_proto, $extra_directives)->to_string;
 }
 
@@ -54,6 +57,11 @@ sub encoded_string { Template::Pure::EncodedString->new($_[1]) }
 sub data_at_path {
   my ($self, $data, $path) = @_;
   my %data_spec = $self->parse_data_spec($path);
+
+  unless(Scalar::Util::blessed($data) and $data->isa('Template::Pure::DataContext') ) {
+    $data = Template::Pure::DataContext->new($data, $self->{root_data});
+  }
+
   return $self->_value_from_data($data, %data_spec);
 }
 
@@ -93,6 +101,8 @@ sub _process_dom_recursive {
       my $value = $self->_value_from_action_proto($dom, $data, $action_proto, %match_spec);
       if(Scalar::Util::blessed($value) && ($value->isa('Template::Pure') || $value->can('TO_HTML') ) ) {
         $self->_process_obj($dom, $data, $value, %match_spec);
+      } elsif((ref($value)||'') eq 'CODE') {
+        $self->_process_code($dom, $data, $value, %match_spec);
       } else {
         $self->_process_match_spec($dom, $value, %match_spec);
       }
@@ -776,6 +786,9 @@ Results in:
 This feature is currently only active for scalar actions but may be extended to other action
 types in the future.
 
+B<NOTE> If your scalar action returns a coderefence, we process it as if the
+scalar action was itself a code reference.  See L<\'Coderef - Programmatically replace the value indicated'>.
+
 =head2 ScalarRef - Set the value to the results of a match
 
 There may be times when you want to set the value of something to an existing
@@ -837,7 +850,8 @@ or:
       directives => [
         '#name' => sub {
           my ($instance, $dom, $data) = @_;
-          return $data->{id}{first_name} .' '. $data->{id}{first_name}; 
+          return $instance->data_at_path($data, 'id.first_name') .' '. 
+            $instance->data_at_path($data, 'id.last_name') ; 
         },
       ]
     );
@@ -868,22 +882,12 @@ arguments:
 Your just need to return the value desired which will substitute for the matched node's
 current value.
 
-B<NOTE>: It might be a good idea to try and maintain as much implementation independence
-from you $data model as possible.  That way if later you change your $data from a hashref
-to an instance of an object you won't break your code.  One way to help achieve this is
-to use L<Template::Pure>'s data lookup helper methods (which support dot notation and more
-as described below.  For example consider re-writing the above example like this:
-
-    my $pure = Template::Pure->new(
-      template => $html,
-      directives => [
-        '#name' => sub {
-          my ($instance, $dom, $data) = @_;
-          return $instance->data_at_path($data, 'id.first_name') .' '. 
-            $instance->data_at_path($data, 'id.last_name') ; 
-        },
-      ]
-    );
+B<NOTE>: Please note in the above example code that we used 'data_at_path' rather than
+dereferenced the $data scalar directly.  This is required since internally we wrap your
+$data in helper objects, so you can't be 100% certain of the actual structure.  In general
+using this method wouldbe a good idea anyway since it lets you achieve an API that is
+complete independent of your actual data structure (this way if you later change from a 
+simple hashref to an object, your code wouldn't break.
 
 =head2 Arrayref - Run directives under a new DOM root
 
@@ -1674,6 +1678,83 @@ path as undefined:
       ...,
     ],
 
+=head2 Defaults in your Data Context
+
+By default there will be a key 'self' in your data context which refers to
+the current instance of your L<Template::Pure>.  This is handy for introspection
+and for subclassing:
+
+    {
+      package Local::Template::Pure::Custom;
+
+      use Moo;
+      extends 'Template::Pure';
+
+      has 'version' => (is=>'ro', required=>1);
+
+      sub time { return 'Mon Apr 11 10:49:42 2016' }
+    }
+
+    my $html_template = qq[
+      <html>
+        <head>
+          <title>Page Title</title>
+        </head>
+        <body>
+          <div id='version'>Version</div>
+          <div id='main'>Test Body</div>
+          <div id='foot'>Footer</div>
+        </body>
+      </html>
+    ];
+
+    my $pure = Local::Template::Pure::Custom->new(
+      version => 100,
+      template=>$html_template,
+      directives=> [
+        'title' => 'meta.title',
+        '#version' => 'self.version',
+        '#main' => 'story',
+        '#foot' => 'self.time',
+      ]
+    );
+
+Results in:
+
+    <html>
+      <head>
+        <title>A subclass</title>
+      </head>
+      <body>
+        <div id="version">100</div>
+        <div id="main">XXX</div>
+        <div id="foot">Mon Apr 11 10:49:42 2016</div>
+      </body>
+    </html>
+
+Creating subclasses of L<Template::Pure> to encapsulate some of the view
+data abd view logic should probably be considered a best practice approach.
+
+B<NOTE> if you create a subclass and want your methods to have access to
+and to modify the DOM, you can return a CODEREF:
+
+    {
+      package Local::Template::Pure::Custom;
+
+      use Moo;
+      extends 'Template::Pure';
+
+      has 'version' => (is=>'ro', required=>1);
+
+      sub time {
+        return sub {
+        my ($self, $dom, $data) = @_;
+        $dom->attr(foo=>'bar');
+        return 'Mon Apr 11 10:49:42 2016';
+        };
+      }
+    }
+
 =head2 Remapping Your Data Context
 
 If the first element of your directives (either at the root of the directives
